@@ -5,8 +5,8 @@ use bench::criterion::{criterion_group, criterion_main, measurement::WallTime, C
 use pypi_types::Requirement;
 use uv_cache::Cache;
 use uv_client::RegistryClientBuilder;
+use uv_python::PythonEnvironment;
 use uv_resolver::Manifest;
-use uv_toolchain::PythonEnvironment;
 
 fn resolve_warm_jupyter(c: &mut Criterion<WallTime>) {
     let runtime = &tokio::runtime::Builder::new_current_thread()
@@ -18,7 +18,7 @@ fn resolve_warm_jupyter(c: &mut Criterion<WallTime>) {
     let venv = PythonEnvironment::from_root("../../.venv", cache).unwrap();
     let client = &RegistryClientBuilder::new(cache.clone()).build();
     let manifest = &Manifest::simple(vec![Requirement::from(
-        pep508_rs::Requirement::from_str("jupyter").unwrap(),
+        pep508_rs::Requirement::from_str("jupyter==1.0.0").unwrap(),
     )]);
 
     let run = || {
@@ -47,7 +47,7 @@ fn resolve_warm_airflow(c: &mut Criterion<WallTime>) {
     let venv = PythonEnvironment::from_root("../../.venv", cache).unwrap();
     let client = &RegistryClientBuilder::new(cache.clone()).build();
     let manifest = &Manifest::simple(vec![
-        Requirement::from(pep508_rs::Requirement::from_str("apache-airflow[all]").unwrap()),
+        Requirement::from(pep508_rs::Requirement::from_str("apache-airflow[all]==2.9.2").unwrap()),
         Requirement::from(
             pep508_rs::Requirement::from_str("apache-airflow-providers-apache-beam>3.0.0").unwrap(),
         ),
@@ -72,6 +72,7 @@ criterion_main!(uv);
 
 mod resolver {
     use anyhow::Result;
+    use chrono::NaiveDate;
     use once_cell::sync::Lazy;
 
     use distribution_types::IndexLocations;
@@ -81,15 +82,16 @@ mod resolver {
     use uv_cache::Cache;
     use uv_client::RegistryClient;
     use uv_configuration::{
-        BuildOptions, Concurrency, ConfigSettings, PreviewMode, SetupPyStrategy,
+        BuildOptions, Concurrency, ConfigSettings, IndexStrategy, PreviewMode, SetupPyStrategy,
     };
     use uv_dispatch::BuildDispatch;
     use uv_distribution::DistributionDatabase;
     use uv_git::GitResolver;
+    use uv_python::PythonEnvironment;
     use uv_resolver::{
-        FlatIndex, InMemoryIndex, Manifest, Options, PythonRequirement, ResolutionGraph, Resolver,
+        FlatIndex, InMemoryIndex, Manifest, OptionsBuilder, PythonRequirement, ResolutionGraph,
+        Resolver,
     };
-    use uv_toolchain::PythonEnvironment;
     use uv_types::{BuildIsolation, EmptyInstalledPackages, HashStrategy, InFlight};
 
     static MARKERS: Lazy<MarkerEnvironment> = Lazy::new(|| {
@@ -126,8 +128,17 @@ mod resolver {
         venv: &PythonEnvironment,
     ) -> Result<ResolutionGraph> {
         let build_isolation = BuildIsolation::Isolated;
+        let build_options = BuildOptions::default();
         let concurrency = Concurrency::default();
         let config_settings = ConfigSettings::default();
+        let exclude_newer = Some(
+            NaiveDate::from_ymd_opt(2024, 6, 20)
+                .unwrap()
+                .and_hms_opt(0, 0, 0)
+                .unwrap()
+                .and_utc()
+                .into(),
+        );
         let flat_index = FlatIndex::default();
         let git = GitResolver::default();
         let hashes = HashStrategy::None;
@@ -137,7 +148,8 @@ mod resolver {
         let installed_packages = EmptyInstalledPackages;
         let interpreter = venv.interpreter();
         let python_requirement = PythonRequirement::from_interpreter(interpreter);
-        let build_options = BuildOptions::default();
+
+        let options = OptionsBuilder::new().exclude_newer(exclude_newer).build();
 
         let build_context = BuildDispatch::new(
             client,
@@ -148,18 +160,20 @@ mod resolver {
             &index,
             &git,
             &in_flight,
+            IndexStrategy::default(),
             SetupPyStrategy::default(),
             &config_settings,
             build_isolation,
             LinkMode::default(),
             &build_options,
+            exclude_newer,
             concurrency,
             PreviewMode::Disabled,
         );
 
         let resolver = Resolver::new(
             manifest,
-            Options::default(),
+            options,
             &python_requirement,
             Some(&MARKERS),
             Some(&TAGS),
