@@ -181,8 +181,8 @@ impl Cache {
 
     /// Create an ephemeral Python environment in the cache.
     pub fn environment(&self) -> io::Result<tempfile::TempDir> {
-        fs::create_dir_all(self.bucket(CacheBucket::Environments))?;
-        tempfile::tempdir_in(self.bucket(CacheBucket::Environments))
+        fs::create_dir_all(self.bucket(CacheBucket::Builds))?;
+        tempfile::tempdir_in(self.bucket(CacheBucket::Builds))
     }
 
     /// Returns `true` if a cache entry must be revalidated given the [`Refresh`] policy.
@@ -388,13 +388,34 @@ impl Cache {
             }
         }
 
-        for entry in fs::read_dir(self.bucket(CacheBucket::Archive))? {
-            let entry = entry?;
-            let path = entry.path().canonicalize()?;
-            if !references.contains(&path) {
-                debug!("Removing dangling cache entry: {}", path.display());
-                summary += rm_rf(path)?;
+        match fs::read_dir(self.bucket(CacheBucket::Archive)) {
+            Ok(entries) => {
+                for entry in entries {
+                    let entry = entry?;
+                    let path = entry.path().canonicalize()?;
+                    if !references.contains(&path) {
+                        debug!("Removing dangling cache entry: {}", path.display());
+                        summary += rm_rf(path)?;
+                    }
+                }
             }
+            Err(err) if err.kind() == io::ErrorKind::NotFound => (),
+            Err(err) => return Err(err),
+        }
+
+        // Third, remove any cached environments. These are never referenced by symlinks, so we can
+        // remove them directly.
+        match fs::read_dir(self.bucket(CacheBucket::Environments)) {
+            Ok(entries) => {
+                for entry in entries {
+                    let entry = entry?;
+                    let path = entry.path().canonicalize()?;
+                    debug!("Removing dangling cache entry: {}", path.display());
+                    summary += rm_rf(path)?;
+                }
+            }
+            Err(err) if err.kind() == io::ErrorKind::NotFound => (),
+            Err(err) => return Err(err),
         }
 
         Ok(summary)
@@ -634,6 +655,8 @@ pub enum CacheBucket {
     /// other buckets directly would make atomic operations impossible.
     Archive,
     /// Ephemeral virtual environments used to execute PEP 517 builds and other operations.
+    Builds,
+    /// Reusable virtual environments used to invoke Python tools.
     Environments,
 }
 
@@ -647,7 +670,8 @@ impl CacheBucket {
             Self::Simple => "simple-v9",
             Self::Wheels => "wheels-v1",
             Self::Archive => "archive-v0",
-            Self::Environments => "environments-v0",
+            Self::Builds => "builds-v0",
+            Self::Environments => "environments-v1",
         }
     }
 
@@ -758,6 +782,9 @@ impl CacheBucket {
             Self::Archive => {
                 // Nothing to do.
             }
+            Self::Builds => {
+                // Nothing to do.
+            }
             Self::Environments => {
                 // Nothing to do.
             }
@@ -775,6 +802,7 @@ impl CacheBucket {
             Self::Interpreter,
             Self::Simple,
             Self::Archive,
+            Self::Builds,
             Self::Environments,
         ]
         .iter()
