@@ -1,4 +1,4 @@
-use distribution_types::InstalledDist;
+use distribution_types::{InstalledDist, InstalledEggInfoFile};
 
 /// Uninstall a package from the specified Python environment.
 pub async fn uninstall(
@@ -8,12 +8,13 @@ pub async fn uninstall(
         let dist = dist.clone();
         move || match dist {
             InstalledDist::Registry(_) | InstalledDist::Url(_) => {
-                install_wheel_rs::uninstall_wheel(dist.path())
+                Ok(install_wheel_rs::uninstall_wheel(dist.path())?)
             }
-            InstalledDist::EggInfo(_) => install_wheel_rs::uninstall_egg(dist.path()),
+            InstalledDist::EggInfoDirectory(_) => Ok(install_wheel_rs::uninstall_egg(dist.path())?),
             InstalledDist::LegacyEditable(dist) => {
-                install_wheel_rs::uninstall_legacy_editable(&dist.egg_link)
+                Ok(install_wheel_rs::uninstall_legacy_editable(&dist.egg_link)?)
             }
+            InstalledDist::EggInfoFile(dist) => Err(UninstallError::Distutils(dist)),
         }
     })
     .await??;
@@ -23,6 +24,8 @@ pub async fn uninstall(
 
 #[derive(thiserror::Error, Debug)]
 pub enum UninstallError {
+    #[error("Unable to uninstall `{0}`. distutils-installed distributions do not include the metadata required to uninstall safely.")]
+    Distutils(InstalledEggInfoFile),
     #[error(transparent)]
     Uninstall(#[from] install_wheel_rs::Error),
     #[error(transparent)]
