@@ -14,7 +14,7 @@ use pep440_rs::Version;
 use platform_tags::{TagCompatibility, Tags};
 use pypi_types::HashDigest;
 use uv_client::FlatIndexEntries;
-use uv_configuration::{NoBinary, NoBuild};
+use uv_configuration::BuildOptions;
 use uv_normalize::PackageName;
 use uv_types::HashStrategy;
 
@@ -36,8 +36,7 @@ impl FlatIndex {
         entries: FlatIndexEntries,
         tags: Option<&Tags>,
         hasher: &HashStrategy,
-        no_build: &NoBuild,
-        no_binary: &NoBinary,
+        build_options: &BuildOptions,
     ) -> Self {
         // Collect compatible distributions.
         let mut index = FxHashMap::default();
@@ -49,8 +48,7 @@ impl FlatIndex {
                 filename,
                 tags,
                 hasher,
-                no_build,
-                no_binary,
+                build_options,
                 url,
             );
         }
@@ -61,15 +59,13 @@ impl FlatIndex {
         Self { index, offline }
     }
 
-    #[allow(clippy::too_many_arguments)]
     fn add_file(
         distributions: &mut FlatDistributions,
         file: File,
         filename: DistFilename,
         tags: Option<&Tags>,
         hasher: &HashStrategy,
-        no_build: &NoBuild,
-        no_binary: &NoBinary,
+        build_options: &BuildOptions,
         index: IndexUrl,
     ) {
         // No `requires-python` here: for source distributions, we don't have that information;
@@ -79,7 +75,7 @@ impl FlatIndex {
                 let version = filename.version.clone();
 
                 let compatibility =
-                    Self::wheel_compatibility(&filename, &file.hashes, tags, hasher, no_binary);
+                    Self::wheel_compatibility(&filename, &file.hashes, tags, hasher, build_options);
                 let dist = RegistryBuiltWheel {
                     filename,
                     file: Box::new(file),
@@ -96,7 +92,7 @@ impl FlatIndex {
             }
             DistFilename::SourceDistFilename(filename) => {
                 let compatibility =
-                    Self::source_dist_compatibility(&filename, &file.hashes, hasher, no_build);
+                    Self::source_dist_compatibility(&filename, &file.hashes, hasher, build_options);
                 let dist = RegistrySourceDist {
                     name: filename.name.clone(),
                     version: filename.version.clone(),
@@ -120,16 +116,10 @@ impl FlatIndex {
         filename: &SourceDistFilename,
         hashes: &[HashDigest],
         hasher: &HashStrategy,
-        no_build: &NoBuild,
+        build_options: &BuildOptions,
     ) -> SourceDistCompatibility {
         // Check if source distributions are allowed for this package.
-        let no_build = match no_build {
-            NoBuild::None => false,
-            NoBuild::All => true,
-            NoBuild::Packages(packages) => packages.contains(&filename.name),
-        };
-
-        if no_build {
+        if build_options.no_build_package(&filename.name) {
             return SourceDistCompatibility::Incompatible(IncompatibleSource::NoBuild);
         }
 
@@ -154,16 +144,10 @@ impl FlatIndex {
         hashes: &[HashDigest],
         tags: Option<&Tags>,
         hasher: &HashStrategy,
-        no_binary: &NoBinary,
+        build_options: &BuildOptions,
     ) -> WheelCompatibility {
         // Check if binaries are allowed for this package.
-        let no_binary = match no_binary {
-            NoBinary::None => false,
-            NoBinary::All => true,
-            NoBinary::Packages(packages) => packages.contains(&filename.name),
-        };
-
-        if no_binary {
+        if build_options.no_binary_package(&filename.name) {
             return WheelCompatibility::Incompatible(IncompatibleWheel::NoBinary);
         }
 
