@@ -1,5 +1,5 @@
+use std::collections::hash_map::Entry;
 use std::str::FromStr;
-use std::sync::Arc;
 
 use rustc_hash::FxHashMap;
 use tracing::trace;
@@ -99,7 +99,7 @@ impl Preference {
 
 /// A set of pinned packages that should be preserved during resolution, if possible.
 #[derive(Debug, Clone, Default)]
-pub struct Preferences(Arc<FxHashMap<PackageName, Pin>>);
+pub struct Preferences(FxHashMap<PackageName, Pin>);
 
 impl Preferences {
     /// Create a map of pinned packages from an iterator of [`Preference`] entries.
@@ -134,7 +134,12 @@ impl Preferences {
             })
             .collect();
 
-        Self(Arc::new(preferences))
+        Self(preferences)
+    }
+
+    /// Return the [`Entry`] for a package in the preferences.
+    pub fn entry(&mut self, package_name: PackageName) -> Entry<PackageName, Pin> {
+        self.0.entry(package_name)
     }
 
     /// Returns an iterator over the preferences.
@@ -168,7 +173,7 @@ impl std::fmt::Display for Preference {
 
 /// The pinned data associated with a package in a locked `requirements.txt` file (e.g., `flask==1.2.3`).
 #[derive(Debug, Clone)]
-struct Pin {
+pub struct Pin {
     version: Version,
     hashes: Vec<HashDigest>,
 }
@@ -184,3 +189,12 @@ impl Pin {
         &self.hashes
     }
 }
+
+impl From<Version> for Pin {
+    fn from(version: Version) -> Self {
+        Self {
+            version,
+            hashes: Vec::new(),
+        }
+    }
+}
