@@ -179,13 +179,15 @@ impl PubGrubRequirement {
                 .map(|specifier| {
                     Locals::map(expected, specifier)
                         .map_err(ResolveError::InvalidVersion)
-                        .and_then(|specifier| Ok(PubGrubSpecifier::try_from(&specifier)?))
+                        .and_then(|specifier| {
+                            Ok(PubGrubSpecifier::from_pep440_specifier(&specifier)?)
+                        })
                 })
                 .fold_ok(Range::full(), |range, specifier| {
                     range.intersection(&specifier.into())
                 })?
         } else {
-            PubGrubSpecifier::try_from(specifier)?.into()
+            PubGrubSpecifier::from_pep440_specifiers(specifier)?.into()
         };
 
         let requirement = Self {
