@@ -18,21 +18,20 @@ use uv_configuration::{
 use uv_configuration::{KeyringProviderType, TargetTriple};
 use uv_dispatch::BuildDispatch;
 use uv_fs::Simplified;
-use uv_git::GitResolver;
 use uv_installer::{SatisfiesResult, SitePackages};
+use uv_python::{
+    EnvironmentPreference, Prefix, PythonEnvironment, PythonRequest, PythonVersion, Target,
+};
 use uv_requirements::{RequirementsSource, RequirementsSpecification};
 use uv_resolver::{
-    DependencyMode, ExcludeNewer, FlatIndex, InMemoryIndex, OptionsBuilder, PreReleaseMode,
-    PythonRequirement, ResolutionMode,
+    DependencyMode, ExcludeNewer, FlatIndex, OptionsBuilder, PreReleaseMode, PythonRequirement,
+    ResolutionMode,
 };
-use uv_toolchain::{
-    EnvironmentPreference, Prefix, PythonEnvironment, PythonVersion, Target, ToolchainRequest,
-};
-use uv_types::{BuildIsolation, HashStrategy, InFlight};
+use uv_types::{BuildIsolation, HashStrategy};
 
 use crate::commands::pip::operations::Modifications;
 use crate::commands::pip::{operations, resolution_environment};
-use crate::commands::{elapsed, ExitStatus};
+use crate::commands::{elapsed, ExitStatus, SharedState};
 use crate::printer::Printer;
 
 /// Install packages into the current environment.
@@ -119,7 +118,7 @@ pub(crate) async fn pip_install(
     let environment = PythonEnvironment::find(
         &python
             .as_deref()
-            .map(ToolchainRequest::parse)
+            .map(PythonRequest::parse)
             .unwrap_or_default(),
         EnvironmentPreference::from_system_flag(system, true),
         &cache,
@@ -199,9 +198,9 @@ pub(crate) async fn pip_install(
                     printer.stderr(),
                     "{}",
                     format!(
-                        "Audited {} in {}",
+                        "Audited {} {}",
                         format!("{num_requirements} package{s}").bold(),
-                        elapsed(start.elapsed())
+                        format!("in {}", elapsed(start.elapsed())).dimmed()
                     )
                     .dimmed()
                 )?;
@@ -243,7 +242,6 @@ pub(crate) async fn pip_install(
 
     // When resolving, don't take any external preferences into account.
     let preferences = Vec::default();
-    let git = GitResolver::default();
 
     // Ignore development dependencies.
     let dev = Vec::default();
@@ -258,12 +256,10 @@ pub(crate) async fn pip_install(
     }
 
     // Initialize the registry client.
-    let client = RegistryClientBuilder::new(cache.clone())
-        .native_tls(native_tls)
-        .connectivity(connectivity)
+    let client = RegistryClientBuilder::from(client_builder)
+        .cache(cache.clone())
         .index_urls(index_locations.index_urls())
         .index_strategy(index_strategy)
-        .keyring(keyring_provider)
         .markers(&markers)
         .platform(interpreter.platform())
         .build();
@@ -285,22 +281,19 @@ pub(crate) async fn pip_install(
         BuildIsolation::Isolated
     };
 
-    // Create a shared in-memory index.
-    let index = InMemoryIndex::default();
+    // Initialize any shared state.
+    let state = SharedState::default();
 
-    // Track in-flight downloads, builds, etc., across resolutions.
-    let in_flight = InFlight::default();
-
-    // Create a build dispatch for resolution.
-    let resolve_dispatch = BuildDispatch::new(
+    // Create a build dispatch.
+    let build_dispatch = BuildDispatch::new(
         &client,
         &cache,
         interpreter,
         &index_locations,
         &flat_index,
-        &index,
-        &git,
-        &in_flight,
+        &state.index,
+        &state.git,
+        &state.in_flight,
         index_strategy,
         setup_py,
         config_settings,
@@ -339,8 +332,8 @@ pub(crate) async fn pip_install(
         python_requirement,
         &client,
         &flat_index,
-        &index,
-        &resolve_dispatch,
+        &state.index,
+        &build_dispatch,
         concurrency,
         options,
         printer,
@@ -358,35 +351,6 @@ pub(crate) async fn pip_install(
         Err(err) => return Err(err.into()),
     };
 
-    // Re-initialize the in-flight map.
-    let in_flight = InFlight::default();
-
-    // If we're running with `--reinstall`, initialize a separate `BuildDispatch`, since we may
-    // end up removing some distributions from the environment.
-    let install_dispatch = if reinstall.is_none() {
-        resolve_dispatch
-    } else {
-        BuildDispatch::new(
-            &client,
-            &cache,
-            interpreter,
-            &index_locations,
-            &flat_index,
-            &index,
-            &git,
-            &in_flight,
-            index_strategy,
-            setup_py,
-            config_settings,
-            build_isolation,
-            link_mode,
-            &build_options,
-            exclude_newer,
-            concurrency,
-            preview,
-        )
-    };
-
     // Sync the environment.
     operations::install(
         &resolution,
@@ -400,9 +364,9 @@ pub(crate) async fn pip_install(
         &hasher,
         &tags,
         &client,
-        &in_flight,
+        &state.in_flight,
         concurrency,
-        &install_dispatch,
+        &build_dispatch,
         &cache,
         &environment,
         dry_run,
