@@ -10,15 +10,15 @@ use unicode_width::UnicodeWidthStr;
 
 use distribution_types::{Diagnostic, InstalledDist, Name};
 use uv_cache::Cache;
+use uv_cli::ListFormat;
 use uv_configuration::PreviewMode;
 use uv_fs::Simplified;
 use uv_installer::SitePackages;
 use uv_normalize::PackageName;
-use uv_toolchain::Toolchain;
-use uv_toolchain::{PythonEnvironment, SystemPython};
+use uv_python::PythonRequest;
+use uv_python::{EnvironmentPreference, PythonEnvironment};
 
 use crate::commands::ExitStatus;
-use crate::commands::ListFormat;
 use crate::printer::Printer;
 
 /// Enumerate the installed packages in the current environment.
@@ -31,18 +31,16 @@ pub(crate) fn pip_list(
     strict: bool,
     python: Option<&str>,
     system: bool,
-    preview: PreviewMode,
+    _preview: PreviewMode,
     cache: &Cache,
     printer: Printer,
 ) -> Result<ExitStatus> {
     // Detect the current Python interpreter.
-    let system = if system {
-        SystemPython::Required
-    } else {
-        SystemPython::Allowed
-    };
-    let environment =
-        PythonEnvironment::from_toolchain(Toolchain::find(python, system, preview, cache)?);
+    let environment = PythonEnvironment::find(
+        &python.map(PythonRequest::parse).unwrap_or_default(),
+        EnvironmentPreference::from_system_flag(system, false),
+        cache,
+    )?;
 
     debug!(
         "Using Python {} environment at {}",
@@ -51,7 +49,7 @@ pub(crate) fn pip_list(
     );
 
     // Build the installed index.
-    let site_packages = SitePackages::from_executable(&environment)?;
+    let site_packages = SitePackages::from_environment(&environment)?;
 
     // Filter if `--editable` is specified; always sort by name.
     let results = site_packages
