@@ -13,16 +13,15 @@ use uv_cache::Cache;
 use uv_client::{BaseClientBuilder, Connectivity};
 use uv_configuration::{KeyringProviderType, PreviewMode};
 use uv_fs::Simplified;
+use uv_python::EnvironmentPreference;
+use uv_python::PythonRequest;
+use uv_python::{Prefix, PythonEnvironment, Target};
 use uv_requirements::{RequirementsSource, RequirementsSpecification};
-use uv_toolchain::EnvironmentPreference;
-use uv_toolchain::ToolchainRequest;
-use uv_toolchain::{Prefix, PythonEnvironment, Target};
 
 use crate::commands::{elapsed, ExitStatus};
 use crate::printer::Printer;
 
 /// Uninstall packages from the current environment.
-#[allow(clippy::too_many_arguments)]
 pub(crate) async fn pip_uninstall(
     sources: &[RequirementsSource],
     python: Option<String>,
@@ -50,7 +49,7 @@ pub(crate) async fn pip_uninstall(
     let environment = PythonEnvironment::find(
         &python
             .as_deref()
-            .map(ToolchainRequest::parse)
+            .map(PythonRequest::parse)
             .unwrap_or_default(),
         EnvironmentPreference::from_system_flag(system, true),
         &cache,
@@ -104,7 +103,7 @@ pub(crate) async fn pip_uninstall(
     let _lock = environment.lock()?;
 
     // Index the current `site-packages` directory.
-    let site_packages = uv_installer::SitePackages::from_executable(&environment)?;
+    let site_packages = uv_installer::SitePackages::from_environment(&environment)?;
 
     // Partition the requirements into named and unnamed requirements.
     let (named, unnamed): (Vec<Requirement>, Vec<UnnamedRequirement<VerbatimParsedUrl>>) = spec
@@ -207,14 +206,14 @@ pub(crate) async fn pip_uninstall(
         printer.stderr(),
         "{}",
         format!(
-            "Uninstalled {} in {}",
+            "Uninstalled {} {}",
             format!(
                 "{} package{}",
                 distributions.len(),
                 if distributions.len() == 1 { "" } else { "s" }
             )
             .bold(),
-            elapsed(start.elapsed())
+            format!("in {}", elapsed(start.elapsed())).dimmed()
         )
         .dimmed()
     )?;
