@@ -17,15 +17,16 @@ use uv_cli::{
     compat::CompatArgs, CacheCommand, CacheNamespace, Cli, Commands, PipCommand, PipNamespace,
     ProjectCommand,
 };
+use uv_cli::{PythonCommand, PythonNamespace, ToolCommand, ToolNamespace};
 #[cfg(feature = "self-update")]
 use uv_cli::{SelfCommand, SelfNamespace};
-use uv_cli::{ToolCommand, ToolNamespace, ToolchainCommand, ToolchainNamespace};
 use uv_configuration::Concurrency;
 use uv_distribution::Workspace;
 use uv_requirements::RequirementsSource;
-use uv_settings::Combine;
+use uv_settings::{Combine, FilesystemOptions};
 
 use crate::commands::ExitStatus;
+use crate::printer::Printer;
 use crate::settings::{
     CacheSettings, GlobalSettings, PipCheckSettings, PipCompileSettings, PipFreezeSettings,
     PipInstallSettings, PipListSettings, PipShowSettings, PipSyncSettings, PipUninstallSettings,
@@ -135,7 +136,7 @@ async fn run() -> Result<ExitStatus> {
     } else if cli.global_args.isolated {
         None
     } else if let Ok(project) = Workspace::discover(&env::current_dir()?, None).await {
-        let project = uv_settings::FilesystemOptions::from_directory(project.root())?;
+        let project = uv_settings::FilesystemOptions::from_directory(project.install_path())?;
         let user = uv_settings::FilesystemOptions::user()?;
         project.combine(user)
     } else {
@@ -148,7 +149,7 @@ async fn run() -> Result<ExitStatus> {
     let globals = GlobalSettings::resolve(&cli.command, &cli.global_args, filesystem.as_ref());
 
     // Resolve the cache settings.
-    let cache_settings = CacheSettings::resolve(cli.cache_args, filesystem.as_ref());
+    let cache_settings = CacheSettings::resolve(*cli.cache_args, filesystem.as_ref());
 
     // Configure the `tracing` crate, which controls internal logging.
     #[cfg(feature = "tracing-durations-export")]
@@ -166,11 +167,11 @@ async fn run() -> Result<ExitStatus> {
 
     // Configure the `Printer`, which controls user-facing output in the CLI.
     let printer = if globals.quiet {
-        printer::Printer::Quiet
+        Printer::Quiet
     } else if globals.verbose > 0 {
-        printer::Printer::Verbose
+        Printer::Verbose
     } else {
-        printer::Printer::Default
+        Printer::Default
     };
 
     // Configure the `warn!` macros, which control user-facing warnings in the CLI.
@@ -215,7 +216,7 @@ async fn run() -> Result<ExitStatus> {
     // Configure the cache.
     let cache = Cache::from_settings(cache_settings.no_cache, cache_settings.cache_dir)?;
 
-    match cli.command {
+    match *cli.command {
         Commands::Pip(PipNamespace {
             command: PipCommand::Compile(args),
         }) => {
@@ -288,7 +289,7 @@ async fn run() -> Result<ExitStatus> {
                 args.settings.link_mode,
                 args.settings.python,
                 args.settings.system,
-                globals.toolchain_preference,
+                globals.python_preference,
                 args.settings.concurrency,
                 globals.native_tls,
                 globals.quiet,
@@ -617,8 +618,8 @@ async fn run() -> Result<ExitStatus> {
             commands::venv(
                 &args.name,
                 args.settings.python.as_deref(),
-                globals.toolchain_preference,
-                globals.toolchain_fetch,
+                globals.python_preference,
+                globals.python_fetch,
                 args.settings.link_mode,
                 &args.settings.index_locations,
                 args.settings.index_strategy,
@@ -636,141 +637,8 @@ async fn run() -> Result<ExitStatus> {
             )
             .await
         }
-        Commands::Project(ProjectCommand::Run(args)) => {
-            // Resolve the settings from the command-line arguments and workspace configuration.
-            let args = settings::RunSettings::resolve(args, filesystem);
-            show_settings!(args);
-
-            // Initialize the cache.
-            let cache = cache.init()?.with_refresh(args.refresh);
-
-            let requirements = args
-                .with
-                .into_iter()
-                .map(RequirementsSource::from_package)
-                .collect::<Vec<_>>();
-
-            commands::run(
-                args.command,
-                requirements,
-                args.package,
-                args.extras,
-                args.dev,
-                args.python,
-                args.settings,
-                globals.isolated,
-                globals.preview,
-                globals.toolchain_preference,
-                globals.toolchain_fetch,
-                globals.connectivity,
-                Concurrency::default(),
-                globals.native_tls,
-                &cache,
-                printer,
-            )
-            .await
-        }
-        Commands::Project(ProjectCommand::Sync(args)) => {
-            // Resolve the settings from the command-line arguments and workspace configuration.
-            let args = settings::SyncSettings::resolve(args, filesystem);
-            show_settings!(args);
-
-            // Initialize the cache.
-            let cache = cache.init()?.with_refresh(args.refresh);
-
-            commands::sync(
-                args.extras,
-                args.dev,
-                args.modifications,
-                args.python,
-                globals.toolchain_preference,
-                globals.toolchain_fetch,
-                args.settings,
-                globals.preview,
-                globals.connectivity,
-                Concurrency::default(),
-                globals.native_tls,
-                &cache,
-                printer,
-            )
-            .await
-        }
-        Commands::Project(ProjectCommand::Lock(args)) => {
-            // Resolve the settings from the command-line arguments and workspace configuration.
-            let args = settings::LockSettings::resolve(args, filesystem);
-            show_settings!(args);
-
-            // Initialize the cache.
-            let cache = cache.init()?.with_refresh(args.refresh);
-
-            commands::lock(
-                args.python,
-                args.settings,
-                globals.preview,
-                globals.toolchain_preference,
-                globals.toolchain_fetch,
-                globals.connectivity,
-                Concurrency::default(),
-                globals.native_tls,
-                &cache,
-                printer,
-            )
-            .await
-        }
-        Commands::Project(ProjectCommand::Add(args)) => {
-            // Resolve the settings from the command-line arguments and workspace configuration.
-            let args = settings::AddSettings::resolve(args, filesystem);
-            show_settings!(args);
-
-            // Initialize the cache.
-            let cache = cache.init()?.with_refresh(args.refresh);
-
-            commands::add(
-                args.requirements,
-                args.editable,
-                args.dependency_type,
-                args.raw_sources,
-                args.rev,
-                args.tag,
-                args.branch,
-                args.extras,
-                args.package,
-                args.python,
-                args.settings,
-                globals.toolchain_preference,
-                globals.toolchain_fetch,
-                globals.preview,
-                globals.connectivity,
-                Concurrency::default(),
-                globals.native_tls,
-                &cache,
-                printer,
-            )
-            .await
-        }
-        Commands::Project(ProjectCommand::Remove(args)) => {
-            // Resolve the settings from the command-line arguments and workspace configuration.
-            let args = settings::RemoveSettings::resolve(args, filesystem);
-            show_settings!(args);
-
-            // Initialize the cache.
-            let cache = cache.init()?;
-
-            commands::remove(
-                args.requirements,
-                args.dependency_type,
-                args.package,
-                args.python,
-                globals.toolchain_preference,
-                globals.toolchain_fetch,
-                globals.preview,
-                globals.connectivity,
-                Concurrency::default(),
-                globals.native_tls,
-                &cache,
-                printer,
-            )
-            .await
+        Commands::Project(project) => {
+            run_project(project, globals, filesystem, cache, printer).await
         }
         #[cfg(feature = "self-update")]
         Commands::Self_(SelfNamespace {
@@ -785,7 +653,7 @@ async fn run() -> Result<ExitStatus> {
             Ok(ExitStatus::Success)
         }
         Commands::Tool(ToolNamespace {
-            command: ToolCommand::Run(args),
+            command: ToolCommand::Run(args) | ToolCommand::Uvx(args),
         }) => {
             // Resolve the settings from the command-line arguments and workspace configuration.
             let args = settings::ToolRunSettings::resolve(args, filesystem);
@@ -802,8 +670,8 @@ async fn run() -> Result<ExitStatus> {
                 args.settings,
                 globals.isolated,
                 globals.preview,
-                globals.toolchain_preference,
-                globals.toolchain_fetch,
+                globals.python_preference,
+                globals.python_fetch,
                 globals.connectivity,
                 Concurrency::default(),
                 globals.native_tls,
@@ -830,8 +698,8 @@ async fn run() -> Result<ExitStatus> {
                 args.force,
                 args.settings,
                 globals.preview,
-                globals.toolchain_preference,
-                globals.toolchain_fetch,
+                globals.python_preference,
+                globals.python_fetch,
                 globals.connectivity,
                 Concurrency::default(),
                 globals.native_tls,
@@ -864,39 +732,39 @@ async fn run() -> Result<ExitStatus> {
             commands::tool_dir(globals.preview)?;
             Ok(ExitStatus::Success)
         }
-        Commands::Toolchain(ToolchainNamespace {
-            command: ToolchainCommand::List(args),
+        Commands::Python(PythonNamespace {
+            command: PythonCommand::List(args),
         }) => {
             // Resolve the settings from the command-line arguments and workspace configuration.
-            let args = settings::ToolchainListSettings::resolve(args, filesystem);
+            let args = settings::PythonListSettings::resolve(args, filesystem);
             show_settings!(args);
 
             // Initialize the cache.
             let cache = cache.init()?;
 
-            commands::toolchain_list(
+            commands::python_list(
                 args.kinds,
                 args.all_versions,
                 args.all_platforms,
-                globals.toolchain_preference,
-                globals.toolchain_fetch,
+                globals.python_preference,
+                globals.python_fetch,
                 globals.preview,
                 &cache,
                 printer,
             )
             .await
         }
-        Commands::Toolchain(ToolchainNamespace {
-            command: ToolchainCommand::Install(args),
+        Commands::Python(PythonNamespace {
+            command: PythonCommand::Install(args),
         }) => {
             // Resolve the settings from the command-line arguments and workspace configuration.
-            let args = settings::ToolchainInstallSettings::resolve(args, filesystem);
+            let args = settings::PythonInstallSettings::resolve(args, filesystem);
             show_settings!(args);
 
             // Initialize the cache.
             let cache = cache.init()?;
 
-            commands::toolchain_install(
+            commands::python_install(
                 args.targets,
                 args.force,
                 globals.native_tls,
@@ -907,55 +775,226 @@ async fn run() -> Result<ExitStatus> {
             )
             .await
         }
-        Commands::Toolchain(ToolchainNamespace {
-            command: ToolchainCommand::Uninstall(args),
+        Commands::Python(PythonNamespace {
+            command: PythonCommand::Uninstall(args),
         }) => {
             // Resolve the settings from the command-line arguments and workspace configuration.
-            let args = settings::ToolchainUninstallSettings::resolve(args, filesystem);
+            let args = settings::PythonUninstallSettings::resolve(args, filesystem);
             show_settings!(args);
 
-            commands::toolchain_uninstall(args.targets, globals.preview, printer).await
+            commands::python_uninstall(args.targets, globals.preview, printer).await
         }
-        Commands::Toolchain(ToolchainNamespace {
-            command: ToolchainCommand::Find(args),
+        Commands::Python(PythonNamespace {
+            command: PythonCommand::Find(args),
         }) => {
             // Resolve the settings from the command-line arguments and workspace configuration.
-            let args = settings::ToolchainFindSettings::resolve(args, filesystem);
+            let args = settings::PythonFindSettings::resolve(args, filesystem);
 
             // Initialize the cache.
             let cache = cache.init()?;
 
-            commands::toolchain_find(
+            commands::python_find(
                 args.request,
-                globals.toolchain_preference,
+                globals.python_preference,
                 globals.preview,
                 &cache,
                 printer,
             )
             .await
         }
-        Commands::Toolchain(ToolchainNamespace {
-            command: ToolchainCommand::Dir,
+        Commands::Python(PythonNamespace {
+            command: PythonCommand::Dir,
         }) => {
-            commands::toolchain_dir(globals.preview)?;
+            commands::python_dir(globals.preview)?;
             Ok(ExitStatus::Success)
         }
     }
 }
 
+/// Run a [`ProjectCommand`].
+async fn run_project(
+    project_command: Box<ProjectCommand>,
+    globals: GlobalSettings,
+    filesystem: Option<FilesystemOptions>,
+    cache: Cache,
+    printer: Printer,
+) -> Result<ExitStatus> {
+    // Write out any resolved settings.
+    macro_rules! show_settings {
+        ($arg:expr) => {
+            if globals.show_settings {
+                writeln!(printer.stdout(), "{:#?}", $arg)?;
+                return Ok(ExitStatus::Success);
+            }
+        };
+        ($arg:expr, false) => {
+            if globals.show_settings {#
+                writeln!(printer.stdout(), "{:#?}", $arg)?;
+            }
+        };
+    }
+
+    match *project_command {
+        ProjectCommand::Run(args) => {
+            // Resolve the settings from the command-line arguments and workspace configuration.
+            let args = settings::RunSettings::resolve(args, filesystem);
+            show_settings!(args);
+
+            // Initialize the cache.
+            let cache = cache.init()?.with_refresh(args.refresh);
+
+            let requirements = args
+                .with
+                .into_iter()
+                .map(RequirementsSource::from_package)
+                .collect::<Vec<_>>();
+
+            commands::run(
+                args.command,
+                requirements,
+                args.package,
+                args.extras,
+                args.dev,
+                args.python,
+                args.settings,
+                globals.isolated,
+                globals.preview,
+                globals.python_preference,
+                globals.python_fetch,
+                globals.connectivity,
+                Concurrency::default(),
+                globals.native_tls,
+                &cache,
+                printer,
+            )
+            .await
+        }
+        ProjectCommand::Sync(args) => {
+            // Resolve the settings from the command-line arguments and workspace configuration.
+            let args = settings::SyncSettings::resolve(args, filesystem);
+            show_settings!(args);
+
+            // Initialize the cache.
+            let cache = cache.init()?.with_refresh(args.refresh);
+
+            commands::sync(
+                args.extras,
+                args.dev,
+                args.modifications,
+                args.python,
+                globals.python_preference,
+                globals.python_fetch,
+                args.settings,
+                globals.preview,
+                globals.connectivity,
+                Concurrency::default(),
+                globals.native_tls,
+                &cache,
+                printer,
+            )
+            .await
+        }
+        ProjectCommand::Lock(args) => {
+            // Resolve the settings from the command-line arguments and workspace configuration.
+            let args = settings::LockSettings::resolve(args, filesystem);
+            show_settings!(args);
+
+            // Initialize the cache.
+            let cache = cache.init()?.with_refresh(args.refresh);
+
+            commands::lock(
+                args.python,
+                args.settings,
+                globals.preview,
+                globals.python_preference,
+                globals.python_fetch,
+                globals.connectivity,
+                Concurrency::default(),
+                globals.native_tls,
+                &cache,
+                printer,
+            )
+            .await
+        }
+        ProjectCommand::Add(args) => {
+            // Resolve the settings from the command-line arguments and workspace configuration.
+            let args = settings::AddSettings::resolve(args, filesystem);
+            show_settings!(args);
+
+            // Initialize the cache.
+            let cache = cache.init()?.with_refresh(args.refresh);
+
+            commands::add(
+                args.requirements,
+                args.editable,
+                args.dependency_type,
+                args.raw_sources,
+                args.rev,
+                args.tag,
+                args.branch,
+                args.extras,
+                args.package,
+                args.python,
+                args.settings,
+                globals.python_preference,
+                globals.python_fetch,
+                globals.preview,
+                globals.connectivity,
+                Concurrency::default(),
+                globals.native_tls,
+                &cache,
+                printer,
+            )
+            .await
+        }
+        ProjectCommand::Remove(args) => {
+            // Resolve the settings from the command-line arguments and workspace configuration.
+            let args = settings::RemoveSettings::resolve(args, filesystem);
+            show_settings!(args);
+
+            // Initialize the cache.
+            let cache = cache.init()?;
+
+            commands::remove(
+                args.requirements,
+                args.dependency_type,
+                args.package,
+                args.python,
+                globals.python_preference,
+                globals.python_fetch,
+                globals.preview,
+                globals.connectivity,
+                Concurrency::default(),
+                globals.native_tls,
+                &cache,
+                printer,
+            )
+            .await
+        }
+    }
+}
+
 fn main() -> ExitCode {
     let result = if let Ok(stack_size) = env::var("UV_STACK_SIZE") {
-        // Artificially limit the stack size to test for stack overflows. Windows has a default stack size of 1MB,
-        // which is lower than the linux and mac default.
+        // Artificially limit or increase the stack size to test without stack overflows in debug
+        // mode. Windows has a default stack size of 1MB, which is lower than the linux and mac
+        // default.
         // https://learn.microsoft.com/en-us/cpp/build/reference/stack-stack-allocations?view=msvc-170
         let stack_size = stack_size.parse().expect("Invalid stack size");
         let tokio_main = move || {
-            tokio::runtime::Builder::new_multi_thread()
+            let runtime = tokio::runtime::Builder::new_multi_thread()
                 .enable_all()
                 .thread_stack_size(stack_size)
                 .build()
-                .expect("Failed building the Runtime")
-                .block_on(run())
+                .expect("Failed building the Runtime");
+            let result = runtime.block_on(run());
+            // Avoid waiting for pending tasks to complete.
+            //
+            // The resolver may have kicked off HTTP requests during resolution that
+            // turned out to be unnecessary. Waiting for those to complete can cause
+            // the CLI to hang before exiting.
+            runtime.shutdown_background();
+            result
         };
         std::thread::Builder::new()
             .stack_size(stack_size)
@@ -964,11 +1003,13 @@ fn main() -> ExitCode {
             .join()
             .expect("Tokio executor failed, was there a panic?")
     } else {
-        tokio::runtime::Builder::new_multi_thread()
+        let runtime = tokio::runtime::Builder::new_multi_thread()
             .enable_all()
             .build()
-            .expect("Failed building the Runtime")
-            .block_on(run())
+            .expect("Failed building the Runtime");
+        let result = runtime.block_on(run());
+        runtime.shutdown_background();
+        result
     };
 
     match result {
